package com.elab.core.componts.chains;

import java.util.List;
import java.util.function.Consumer;

/**
 * 通用的组件处理链
 *
 * @param <I> 处理的接口handler类型
 * @param <T> 入参类型
 */
public class ComponentConsumerHandlerChain<I, T> {

    private final List<I> handlerList;
    private final Consumer<T> consumer;

    private int currentHandlerIndex = -1;

    public ComponentConsumerHandlerChain(List<I> handlerList, Consumer<T> consumer) {
        this.handlerList = handlerList;
        this.consumer = consumer;
    }

    public void run(T req, ChainConsumer<I, T> chainConsumer) {

        if (this.currentHandlerIndex == this.handlerList.size() - 1) {
            this.consumer.accept(req);
            return;
        }

        I handler = this.handlerList.get(++this.currentHandlerIndex);
        chainConsumer.run(handler, req, this);
    }

}