package com.elab.core.componts.chains;

import java.util.List;
import java.util.function.Supplier;

/**
 * 通用的组件处理链
 *
 * @param <I> 处理的接口handler类型
 * @param <T> 入参类型
 * @param <R> 出参类型
 */
public class ComponentSupplierHandlerChain<I, T, R> {

    protected final List<I> handlerList;
    protected final Supplier<R> supplier;

    private int currentHandlerIndex = -1;

    public ComponentSupplierHandlerChain(List<I> handlerList, Supplier<R> supplier) {
        this.handlerList = handlerList;
        this.supplier = supplier;
    }

    public R executor(T req, ChainSupplier<I, T, R> chainFunction) {

        if (this.currentHandlerIndex == this.handlerList.size() - 1) {
            return supplier.get();
        }

        I handler = this.handlerList.get(++this.currentHandlerIndex);
        return chainFunction.executor(handler, req, this);
    }

}