package com.elab.mongodb.chain;

import com.alibaba.fastjson.JSON;
import com.dianping.cat.Cat;
import com.elab.core.componts.chains.ComponentConsumerHandlerChain;
import com.elab.core.componts.chains.ComponentSupplierHandlerChain;
import com.elab.log.utils.CatCrossProcess;
import com.elab.mongodb.chain.model.MongoExecutorRequest;
import com.elab.mongodb.chain.model.MongoInsertRequest;
import com.elab.mongodb.chain.model.MongoQueryRequest;
import com.elab.mongodb.chain.model.MongoUpdateRequest;
import com.mongodb.DBObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.mongodb.core.MongoActionOperation;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;

import java.util.List;

/**
 * mongo拦截器
 *
 * @author liukaixiong
 * @date 2022/8/13 - 16:35
 */
public class CatMongoInterceptInvoke implements MongoInterceptInvoke {

    private final Logger logger = LoggerFactory.getLogger(getClass());

    @Override
    public <T> T execute(MongoExecutorRequest request,
        ComponentSupplierHandlerChain<MongoInterceptInvoke, MongoExecutorRequest, T> chain) {
        //        String collectionName = request.getCollectionName();
        //        try {
        //            String type = String.format("%s - %s", collectionName, MongoActionOperation.INSERT.name());
        //            return CatCrossProcess.processTransactionMsg("mongodb", type, (t) -> {
        //                logger.info("---------->>>>>>>>>>>>>>>" + Cat.getCurrentMessageId());
        //                T executor = chain.executor(request, MongoInterceptInvoke::execute);
        //                logger.info("mongo result : {}", getJsonString(executor));
        //                return executor;
        //            });
        //        } catch (Exception e) {
        //            logger.error("mongodb error", e);
        //        }
        return chain.executor(request, MongoInterceptInvoke::execute);
    }

    @Override
    public void insert(MongoInsertRequest request,
        ComponentConsumerHandlerChain<MongoInterceptInvoke, MongoInsertRequest> chain) {
        String collectionName = request.getCollectionName();
        Object objectToSave = request.getObjectToSave();
        try {
            String type = String.format("%s - %s", collectionName, MongoActionOperation.INSERT.name());
            CatCrossProcess.processTransactionMsg("mongodb", type, (t) -> {
                logger.info("请求入参: " + JSON.toJSONString(objectToSave));
                logger.info("---------->>>>>>>>>>>>>>>" + Cat.getCurrentMessageId());
                chain.run(request, MongoInterceptInvoke::insert);
                return null;
            });
        } catch (Exception e) {
            logger.error("mongo insert error", e);
        }
    }

    @Override
    public <T> T update(MongoUpdateRequest request,
        ComponentSupplierHandlerChain<MongoInterceptInvoke, MongoUpdateRequest, T> chain) {
        String collectionName = request.getCollectionName();
        Query query = request.getQuery();
        Update update = request.getUpdate();

        try {
            String type = String.format("%s - %s", collectionName, MongoActionOperation.UPDATE.name());
            return CatCrossProcess.processTransactionMsg("mongodb", type, (t) -> {
                logger.info("---------->>>>>>>>>>>>>>>" + Cat.getCurrentMessageId());
                logger.debug(" query : {}", getString(query));
                logger.debug(" update :  {} ", getString(update));
                T executor = chain.executor(request, MongoInterceptInvoke::update);
                logger.info("mongo result : {}", getJsonString(executor));
                return executor;
            });
        } catch (Exception e) {
            logger.error("mongodb error", e);
        }
        return null;
    }

    @Override
    public <T> List<T> queryList(MongoQueryRequest request,
        ComponentSupplierHandlerChain<MongoInterceptInvoke, MongoQueryRequest, List<T>> chain) {
        String collectionName = request.getCollectionName();
        DBObject query = request.getQuery();
        DBObject fields = request.getFields();
        try {
            String queryList = "QueryList";
            String type = String.format("%s - %s", collectionName, queryList);
            return CatCrossProcess.processTransactionMsg("mongodb", type, (t) -> {
                logger.debug(" field :  {} ", getString(query));
                logger.debug(" field :  {} ", getString(fields));
                logger.info("---------->>>>>>>>>>>>>>>" + Cat.getCurrentMessageId());
                List<T> executor = chain.executor(request, MongoInterceptInvoke::queryList);
                logger.info("mongo result size : {} ", executor.size());
                return executor;
            });
        } catch (Exception e) {
            logger.error("mongodb error", e);
        }
        return null;
    }

    @Override
    public <T> T queryOne(MongoQueryRequest request,
        ComponentSupplierHandlerChain<MongoInterceptInvoke, MongoQueryRequest, T> chain) {

        String collectionName = request.getCollectionName();
        DBObject query = request.getQuery();
        DBObject fields = request.getFields();
        try {
            String queryObject = "QueryObject";
            String type = String.format("%s - %s", collectionName, queryObject);
            return CatCrossProcess.processTransactionMsg("mongodb", type, (t) -> {
                logger.debug(" query : {}", getString(query));
                logger.debug(" field :  {} ", getString(fields));
                logger.info("---------->>>>>>>>>>>>>>>" + Cat.getCurrentMessageId());
                T executor = chain.executor(request, MongoInterceptInvoke::queryOne);
                logger.info("mongo result : {}", getJsonString(executor));
                return executor;
            });
        } catch (Exception e) {
            logger.error("mongodb error", e);
        }
        return null;
    }

    private String getString(Object obj) {
        if (obj == null) {
            return "";
        }
        return obj.toString();
    }

    private String getJsonString(Object obj) {
        if (obj == null) {
            return "";
        } else if (obj instanceof String) {
            return (String)obj;
        }
        try {
            int maxLength = 1000;
            String result = JSON.toJSONString(obj);
            // 优化日志输出
            if (result.length() > maxLength) {
                return result.substring(0, maxLength);
            }
            return result;
        } catch (Exception e) {
            logger.warn("格式转换异常:{}", e.getMessage());
        }
        return "";
    }
}
