package com.elab.mongodb.chain.example;

import java.util.Arrays;
import java.util.List;
import java.util.function.Supplier;

/**
 * 通用责任链拦截器
 *
 * @author liukaixiong
 * @date 2022/8/13 - 16:54
 */
public class UserRecordHandlerChain<I, T, R> {

    private final List<I> handlerList;
    private final Supplier<R> supplier;

    private int currentHandlerIndex = -1;

    public UserRecordHandlerChain(List<I> handlerList, Supplier<R> supplier) {
        this.handlerList = handlerList;
        this.supplier = supplier;
    }

    public R executor(T req, ChainFunction<I, T, R> chainFunction) {

        if (this.currentHandlerIndex == this.handlerList.size() - 1) {
            return supplier.get();
        }

        I handler = this.handlerList.get(++this.currentHandlerIndex);
        return chainFunction.executor(handler, req, this);
    }

    public static class UserModel {
        private String username;
        private int age;

        public String getUsername() {
            return username;
        }

        public void setUsername(String username) {
            this.username = username;
        }

        public int getAge() {
            return age;
        }

        public void setAge(int age) {
            this.age = age;
        }

        @Override
        public String toString() {
            return "UserModel{" + "username='" + username + '\'' + ", age=" + age + '}';
        }
    }

    interface ChainFunction<I, T, R> {
        public R executor(I apply, T req, UserRecordHandlerChain<I, T, R> chain);
    }

    class DefaultChainFunction implements ChainFunction<Handler, UserModel, String> {
        @Override
        public String executor(Handler apply, UserModel userModel,
            UserRecordHandlerChain<Handler, UserModel, String> chain) {
            return apply.run(userModel, chain);
        }
    }

    /**
     * 通用的拦截链处理器
     */
    interface Handler {
        public String run(UserModel userModel, UserRecordHandlerChain<Handler, UserModel, String> chain);
    }

    static class AHandler implements Handler {
        @Override
        public String run(UserModel userModel, UserRecordHandlerChain<Handler, UserModel, String> chain) {
            System.out.println("A 开始"+userModel.toString());
            String executor = chain.executor(userModel, Handler::run);
            System.out.println("A 结束");
            return executor;
        }
    }

    static class BHandler implements Handler {
        @Override
        public String run(UserModel userModel, UserRecordHandlerChain<Handler, UserModel, String> chain) {
            System.out.println("B 开始 "+userModel.toString());
            String executor = chain.executor(userModel, Handler::run);
            System.out.println("B 结束");
            return executor;
        }
    }

    public static void main(String[] args) {
        List<Handler> handlers = Arrays.asList(new AHandler(), new BHandler());
        UserRecordHandlerChain<Handler, UserModel, String> chain =
            new UserRecordHandlerChain<Handler, UserModel, String>(handlers, () -> {
                System.out.println("ok");
                return "ok";
            });

        UserModel user = new UserModel();
        user.setAge(1);
        user.setUsername("liukx");

        String executor = chain.executor(user, Handler::run);
        System.out.println("得到结果:" + executor);

    }
}
